package org.khelekore.jsizer;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

/** Class to handle coloring of nodes.
 */
public class ColorHandler {
    private SizeHandler sizeHandler;

    // class name -> base color
    private Map<String, Color> colorMap = new HashMap<String, Color> ();
    // cache to reuse colors.
    private Map<Color, Color> colorCache = new HashMap<Color, Color> ();

    public ColorHandler (SizeHandler sizeHandler) {
	this.sizeHandler = sizeHandler;
    }

    public void addColor (String classname, Color color) {
	colorMap.put (classname, color);
    }

    public void removeColor (String classname) {
	colorMap.remove (classname);
    }

    public void clearColors () {
	colorMap.clear ();
    }

    public Color getCachedColor (Color color) {
	Color cc = colorCache.get (color);
	if (cc == null) {
	    colorCache.put (color, color);
	    cc = color;
	}
	return cc;
    }

    public Color getColor (int numLines) {
	int maxLines = sizeHandler.getMaxLines ();
	float c = (float)(.9 - (numLines / (double)maxLines) * .9);
	return getCachedColor (new Color (c, c, c));
    }

    public Color getColor (ClassSize cs) {
	Color b = getColor (cs.getNumMethodLines ());
	if (!colorMap.isEmpty ()) {
	    String cn = cs.getClassName ();
	    Color c = colorMap.get (cn);
	    int i = cn.lastIndexOf ('$');
	    // class name has to have at least one char before $
	    while (c == null && i > 0) {
		cn = cn.substring (0, i);
		c = colorMap.get (cn);
		i = cn.lastIndexOf ('$');
	    }
	    if (c != null) {
		int k1 = c.getRed ();
		double x = (255.0 - k1) / 255;
		int rc = (int)(b.getRed () * x + k1);
		int k2 = c.getGreen ();
		double y = (255.0 - k2) / 255;
		int gc = (int)(b.getGreen () * y + k2);
		int k3 = c.getBlue ();
		double z = (255.0 - k3) / 255;
		int bc = (int)(b.getBlue () * z + k3);
		return getCachedColor (new Color (rc, gc, bc));
	    }
	}
	return b;
    }
}
